''*****************************************************************************
''
''	Platformer tutorial 101
''
''	Chapter 02-B
''  Handling Actions and Animations with Finite State Machines
''
''	Relminator (Richard Eric M. Lope)
''	http://rel.phatcode.net
''
''	Entity management using Finite State Machines(FSM)
''	FSM is a programming paradigm where there can only
''	be 1 state for an entity in a given time.
''
''	FSMs make entity management easier because it is easy to
''	visualize logic by limiting action to a single state
''	in a given frame(or time).
''
''	Code may be a little bigger but the consequences of
''	not using it( if FSMs is suitable ) is very messy 
''	as I've learned the hard way
''
''  I'm using EASY GL2D for rendering.
'' 	So try to read these tutorials for Easy GL2D..
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_1
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_2

''*****************************************************************************

#include once "fbgfx.bi"
#include once "/gl/gl.bi" 
#include once "/gl/glu.bi"   

#include once "FBGL2D.bi"     	'' We're gonna use Hardware acceleration
#include once "FBGL2D.bas"		'' So we'll be using my LIB



''*****************************************************************************
const as integer SCREEN_WIDTH = 640
const as integer SCREEN_HEIGHT = 480


const as integer FALSE = 0
const as integer TRUE = not FALSE

const as single GRAVITY = 0.75
const as single JUMPHEIGHT = 15

const as integer FLOOR_VALUE = 400  '' Our arbitrary floor value for collision purposes


''*****************************************************************************
''
''	Our Player Class
''
''*****************************************************************************
Type Player

public:
	
	'' Player States for Finite State Machine
	Enum E_STATE
		IDLE = 0,
		WALKING,
		JUMPING,
		FALLING,
		BORED,
		PLANT_BOMB,
		LIGHT_DYNAMITE,
		DIED
	End Enum
	
	'' Direction our player is facing for flipmode
	Enum E_DIRECTION
		DIR_RIGHT = 0,
		DIR_LEFT,
		DIR_UP,
		DIR_DOWN	
	End Enum
	
	'' length of time before the player gets "bored" standing up 
	Enum 
		BORED_WAIT_TIME = 60 * 3
	End Enum
	
	declare constructor()
	declare destructor()
	
	declare Sub Update()			'' This updates the player depending on its State
	declare sub Draw(SpriteSet() as GL2D.IMAGE ptr)				'' Draws the Player according to state
	
private:
	
	declare Sub ActionIdle()			''\ 
	declare Sub ActionWalking()			'' |  
	declare Sub ActionJumping()			'' | 
	declare Sub ActionBored()			''  \ These are the functions to be called by Update
	declare Sub ActionFalling()			''  / Depending on the Player.State
	declare Sub ActionLightDynamite()	'' |
	declare Sub ActionPlantBomb()		'' |
	declare Sub ActionDied()			''/
	
	declare sub ResolveAnimationParameters()	'' Sets up animation params depending on State
	declare sub Animate()						'' Animates the player
	
	x		as single				'' Position
	y		as single
	Dx		as single				'' Direction
	Dy		as Single
	Speed	as single				'' Horizontal Speed
	Wid		as integer				'' Width of the player
	Hei		as integer				
	CanJump as integer				'' If the player can Jump
	
	Counter 	as Integer			'' Arbitrary counter for animations
	Frame		as Integer			'' Sprite Frame
	BaseFrame 	as Integer			'' Starting Sprite frame see "gripe.bmp"
	MaxFrame	as Integer			'' Number of animation frames
	FlipMode	as Integer			'' Whether sprite faces left or right
	
	Direction 		as Integer		'' Direction our player faces
	StandingCounter as Integer		'' Counter for BORED state
	
	State 			as E_STATE				'' State of the player
	
End Type

''*****************************************************************************
'' Set up player
''*****************************************************************************
constructor Player()
	
	Counter = 0
	x = 32 * 2 
	y = 32 * 5 
	Speed = 2.5
	Wid = 24
	Hei = 32
	CanJump = false
	
	Frame = 0
	BaseFrame = 0
	MaxFrame = 8	
	FlipMode = GL2D.FLIP_NONE
	
	Direction = DIR_RIGHT
	
	State = FALLING
	StandingCounter = 0
	ResolveAnimationParameters()
	
End Constructor

''*****************************************************************************
''
''*****************************************************************************
destructor Player()

End Destructor



''*****************************************************************************
'' Updates the player according to states
''*****************************************************************************
sub Player.Update()

	Counter += 1
	Animate()		'' Animate player
	
	'' Check to see the Player State and
	'' Update accordingly
	
	'' In my DS game I used function pointers 
	'' But I have no idea how to get and call
	'' function pointers of class methods in FB
	'' so I used select case which is not too bad.
	Select Case State
		case IDLE:
			ActionIdle()
		case WALKING:
			StandingCounter = 0    '' Set bored counter to zero since we don't want to get bored while walking
			ActionWalking()
		case JUMPING:
			StandingCounter = 0
			ActionJumping()
		case FALLING:
			StandingCounter = 0
			ActionFalling()
		case BORED:
			StandingCounter = 0
			ActionBored()
		case LIGHT_DYNAMITE:
			StandingCounter = 0
			ActionLightDynamite()
		case PLANT_BOMB:
			StandingCounter = 0
			ActionPlantBomb()
		case DIED:
			StandingCounter = 0
			ActionDied()
	End Select
    
    
End Sub

''*****************************************************************************
'' Called when player is not moving and standing on the ground
''*****************************************************************************
Sub Player.ActionIdle()
	
	Dx = 0  '' Set speed to 0
	
	'' Did we dilly-dally too long?
	'' If so, get "BORED"
	StandingCounter += 1
	if( StandingCounter > BORED_WAIT_TIME ) then
		State = BORED
		ResolveAnimationParameters()
	EndIf
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
		Direction = DIR_LEFT
		ResolveAnimationParameters()
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    	Direction = DIR_RIGHT
		ResolveAnimationParameters()
    EndIf
    
    '' We can Light a dynamite while standing
    if multikey(FB.SC_Z) then 
    	State = LIGHT_DYNAMITE
		ResolveAnimationParameters()
    EndIf
    
    '' We can plant bombs while standing
    if multikey(FB.SC_X) then 
    	State = PLANT_BOMB
		ResolveAnimationParameters()
    EndIf
    
    '' We can die while standing
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    '' We can jump while not moving so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump again while in the middle of Jumping
	    	ResolveAnimationParameters()
	    end if
    EndIf
    
	
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei > FLOOR_VALUE ) then    
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true		'' We hit floor so we can jump again
	End If
		
	
End Sub

''*****************************************************************************
'' Called when player is walking on the ground
''*****************************************************************************
Sub Player.ActionWalking()
	
	'' Assume we are not moving so set
	'' State to be idle
	State = IDLE
	
	Dx = 0  '' Set speed to 0
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
		Direction = DIR_LEFT
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    	Direction = DIR_RIGHT
    EndIf
    
    '' We can Light a dynamite while walking
    if multikey(FB.SC_Z) then 
    	State = LIGHT_DYNAMITE
		ResolveAnimationParameters()
    EndIf
    
    '' We can plant bombs while walking
    if multikey(FB.SC_X) then 
    	State = PLANT_BOMB
		ResolveAnimationParameters()
    EndIf
    
    '' We can die while walking
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    
    '' We can jump while walking so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump while Jumping
	    	ResolveAnimationParameters()
	    end if
    EndIf
    
	
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei > FLOOR_VALUE ) then    
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true		'' We hit floor so we can jump again
	End If
	
	'' If we did not move, change animation frames	
	if( State = IDLE ) then ResolveAnimationParameters()
	
End Sub

''*****************************************************************************
'' Called when player is jumping
''*****************************************************************************
Sub Player.ActionJumping()
	
	'' You will notice that there is no way to plant bombs or dynamite within this sub
	'' This is the beauty of FSM. You can limit behaviors depending on your needs.
	'' I didn't want the player to plant bombs or dynamites while jumping or falling so
	'' I just didn't include a check here.
	
	dim as integer Walked = FALSE    '' a way to check if we moved left or right
									 '' Since Dx is single and EPSILON would not look
									 '' good in a tutorial
	
	Dx = 0  '' Set speed to 0
	'' We can move left or right when jumping so...
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		Walked = TRUE  			''Set walked to true for checking later 
		Direction = DIR_LEFT
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	Walked = TRUE
    	Direction = DIR_RIGHT
    EndIf
    
    '' We can die while jumping
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei > FLOOR_VALUE ) then
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true	'' We hit floor so we can jump again
		
		'' Check if we walked or not
		if( Walked ) then
			State = WALKING	'' Set the State to WALKING when we hit the floor
			ResolveAnimationParameters()
		else
			State = IDLE	'' Ditto
			ResolveAnimationParameters()
		EndIf
		
	End If
	
End Sub

''*****************************************************************************
'' Called when player is falling
''*****************************************************************************
Sub Player.ActionFalling()
	
	'' You will notice that there is no way to plant bombs or dynamite within this sub
	'' This is the beauty of FSM. You can limit behaviors depending on your needs.
	'' I didn't want the player to plant bombs or dynamites while jumping or falling so
	'' I just didn't include a check here.
	
	dim as integer Walked = FALSE    '' a way to check if we moved left or right
									 '' Since Dx is single and EPSILON would not look
									 '' good in a tutorial
	
	Dx = 0  '' Set speed to 0
	'' We can move left or right when falling so...
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		Walked = TRUE  			''Set walked to true for checking later 
		Direction = DIR_LEFT
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	Walked = TRUE
    	Direction = DIR_RIGHT
    EndIf
    
    '' We can die while falling
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei > FLOOR_VALUE ) then
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true	'' We hit floor so we can jump again
		
		'' Check if we walked or not
		if( Walked ) then
			State = WALKING	'' Set the State to WALKING when we hit the floor
			ResolveAnimationParameters()
		else
			State = IDLE	'' Ditto
			ResolveAnimationParameters()
		EndIf
		
	End If
	
End Sub

''*****************************************************************************
'' Called when Player gets bored standing up for a while
''*****************************************************************************
Sub Player.ActionBored()
	
	Dx = 0  '' Set speed to 0
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
		Direction = DIR_LEFT
		ResolveAnimationParameters()
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    	Direction = DIR_RIGHT
		ResolveAnimationParameters()
    EndIf
    
    '' We can Light a dynamite while bored
    if multikey(FB.SC_Z) then 
    	State = LIGHT_DYNAMITE
		ResolveAnimationParameters()
    EndIf
    
    '' We can plant bombs while bored
    if multikey(FB.SC_X) then 
    	State = PLANT_BOMB
		ResolveAnimationParameters()
    EndIf
    
    '' We can die while being bored
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
    '' We can jump while not moving so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump while Jumping
	    	ResolveAnimationParameters()
	    end if
    EndIf
    
	
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei > FLOOR_VALUE ) then    
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true		'' We hit floor so we can jump again
	End If
	
	
End Sub


''*****************************************************************************
'' Called when Player lights up a dynamite
''*****************************************************************************
Sub Player.ActionLightDynamite()
	
	
	'' We can die while lighting a dynamite
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
	'' We don't move while lighting up a dynamite
	'' We can only light a dynamite if we are either
	'' STANDING, WALKING, BORED
	'' We can't light a Dynamite while Jumping
	Dx = 0  '' Set speed to 0
	Dy = 0	
	if( Frame = (MaxFrame-1) ) then
		State = IDLE
		ResolveAnimationParameters()
	EndIf
	
End Sub

''*****************************************************************************
'' Called when Player lights up a dynamite
''*****************************************************************************
Sub Player.ActionPlantBomb()
	
	'' We can die while planting bombs
    if multikey(FB.SC_C) then 
    	State = DIED
    	Dy = -JUMPHEIGHT	'' Mario Style death		
		ResolveAnimationParameters()
    EndIf
   
   '' We don't move while Planting Bombs
	'' We can only Plant bombs if we are either
	'' STANDING, WALKING, BORED
	'' We can't Plant bombs while Jumping
	Dx = 0  '' Set speed to 0
	Dy = 0	
	if( Frame = (MaxFrame-1) ) then
		State = IDLE
		ResolveAnimationParameters()
	EndIf
	
End Sub


''*****************************************************************************
'' Called when player is dead
''*****************************************************************************
Sub Player.ActionDied()

	'' We can't do anything except fall down when dying so no checks for
	'' walking, jumping, dynamite, bombs, etc.
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	y += Dy		'' Move
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei > FLOOR_VALUE ) then
		Dy = 0
		y = FLOOR_VALUE - Hei
		
		State = IDLE			'' Set to standing for now when we hit the floor while dying
		ResolveAnimationParameters()
	
	End If
	

End Sub


''*****************************************************************************
'' Draws the player according to state
''*****************************************************************************
sub Player.Draw(SpriteSet() as GL2D.IMAGE ptr)

	Select Case State
		case IDLE:
			GL2D.PrintScale(0, 0, 1, "State = IDLE" )
		case WALKING:
			GL2D.PrintScale(0, 0, 1, "State = WALKING" )
		case JUMPING:
			GL2D.PrintScale(0, 0, 1, "State = JUMPING" )
		case FALLING:
			GL2D.PrintScale(0, 0, 1, "State = FALLING" )
		case BORED:
			GL2D.PrintScale(0, 0, 1, "State = BORED" )
		case LIGHT_DYNAMITE:
			GL2D.PrintScale(0, 0, 1, "State = LIGHT_DYNAMITE" )
		case PLANT_BOMB:
			GL2D.PrintScale(0, 0, 1, "State = PLANT_BOMB" )
		case DIED:
			GL2D.PrintScale(0, 0, 1, "State = DIED" )
	End Select
		
		GL2D.SpriteFlip( x-2, y, FlipMode, SpriteSet(BaseFrame + Frame))
			
		GL2D.PrintScale(0, 20, 1, "Bored counter = " & str(StandingCounter) )    
	
		GL2D.PrintScale(0, 30, 1, "Bored wait = " & str(BORED_WAIT_TIME) )    
	
		GL2D.PrintScale(0,  50, 1, "CONTROLS")    
		GL2D.PrintScale(0,  70, 1, "ARROWS = Move")    
		GL2D.PrintScale(0,  80, 1, " SPACE = Jump")    
		GL2D.PrintScale(0,  90, 1, "     Z = Light Dynamite")    
		GL2D.PrintScale(0, 100, 1, "     X = Plant Bomb")
		GL2D.PrintScale(0, 110, 1, "     C = Die")    
	    
	    GL2D.PrintScale(0, 130, 1, "Sprites by Marc Russel (SpicyPixel.Net)")    
	    
	
end sub

''*****************************************************************************
'' Animates the player
''*****************************************************************************
Sub Player.Animate()
	
	'' Only animate every 4 frames
	If( (Counter and 3) = 0 ) Then
		Frame = ( Frame + 1 ) mod MaxFrame
	EndIf
	
	'' Check which direction the player is facing and
	'' set FlipMode accordingly
	if( Direction = DIR_RIGHT ) then
		FlipMode = GL2D.FLIP_NONE
	else
		FlipMode = GL2D.FLIP_H
	EndIf
	
End Sub

''*****************************************************************************
'' Sets up animation frames depending on current state
''*****************************************************************************
sub Player.ResolveAnimationParameters()
	
	'' See "gripe.bmp" for details
	Select Case State
		case IDLE:
			Frame = 0
			BaseFrame = 16
			MaxFrame = 1	
		case WALKING:
			Frame = 0
			BaseFrame = 0
			MaxFrame = 8
		case JUMPING:
			Frame = 0
			BaseFrame = 17
			MaxFrame = 1
		case FALLING:
			Frame = 0
			BaseFrame = 17
			MaxFrame = 1
		case BORED:
			Frame = 0
			BaseFrame = 9
			MaxFrame = 7
		case LIGHT_DYNAMITE:
			Frame = 0
			BaseFrame = 18
			MaxFrame = 7   '' offset by 1 so that all the frames get drawn
						   '' for we will change states after animation is done
		case PLANT_BOMB:
			Frame = 0
			BaseFrame = 24
			MaxFrame = 5   '' offset by 1 so that all the frames get drawn
						   '' for we will change states after animation is done
		case DIED:
			Frame = 0
			BaseFrame = 28
			MaxFrame = 4   
		End Select
    
End Sub
	
''*****************************************************************************
'' Our main sub
''*****************************************************************************
sub main()

	
	'' Set up a dynamic memory for our images
	redim as GL2D.IMAGE ptr GripeImages(0)
	
	
	dim as integer Frame = 0
	 
	dim as Player Gripe			'' Our player
	 
	GL2D.ScreenInit( SCREEN_WIDTH, SCREEN_HEIGHT )   ''Set up GL screen
	GL2D.VsyncON()
	
	'' Initialize our sprites
	'' See tutorial for Easy GL2D..
	'' http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_1
	'' http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_2
	GL2D.InitSprites( GripeImages(), 32,32, "gripe.bmp", GL_NEAREST )

	do
		
		Frame += 1

		Gripe.Update()     '' Do player movements
    
		GL2D.ClearScreen()
			
		GL2D.Begin2D()
			
			'' Draw floor
			GL2D.BoxFilled( 0, FLOOR_VALUE, SCREEN_WIDTH, SCREEN_HEIGHT, GL2D_RGBA(255,255,255,255) )
			
			'' Draw gripe
			Gripe.Draw(GripeImages())
			
			
		GL2D.End2D()
		
		
		dim as integer FPS = GL2D.LimitFPS(60)
		
		sleep 1,1
		flip
		
	Loop until multikey(FB.SC_ESCAPE)

	GL2D.DestroySprites( GripeImages() )

	GL2D.ShutDown()
	
End Sub

''*****************************************************************************
''*****************************************************************************


main()


end


