''*****************************************************************************
''
''	Platformer tutorial 101
''
''	Chapter 02-A
''  Finite State Machines
''
''	Relminator (Richard Eric M. Lope)
''	http://rel.phatcode.net
''
''	Entity management using Finite State Machines(FSM)
''	FSM is a programming paradigm where there can only
''	be 1 state for an entity in a given time.
''
''	FSMs make entity management easier because it is easy to
''	visualize logic by limiting action to a single state
''	in a given frame(or time).
''
''	Code may be a little bigger but the consequences of
''	not using it( if FSMs is suitable ) is very messy 
''	as I've learned the hard way
''
''  I'm using EASY GL2D for rendering.
'' 	So try to read these tutorials for Easy GL2D..
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_1
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_2
''
''*****************************************************************************

#include once "fbgfx.bi"
#include once "/gl/gl.bi" 
#include once "/gl/glu.bi"   

#include once "FBGL2D.bi"     	'' We're gonna use Hardware acceleration
#include once "FBGL2D.bas"		'' So we'll be using my LIB



''*****************************************************************************
const as integer SCREEN_WIDTH = 640
const as integer SCREEN_HEIGHT = 480


const as integer FALSE = 0
const as integer TRUE = not FALSE

const as single GRAVITY = 0.75
const as single JUMPHEIGHT = 15

const as integer FLOOR_VALUE = 400  '' Our arbitrary floor value for collision purposes


''*****************************************************************************
''
''	Our Player Class
''
''*****************************************************************************
Type Player

public:
	
	'' Player States for our Finite State Machine
	Enum E_STATE
		IDLE = 0,
		WALKING,
		JUMPING
	End Enum
	
	
	declare constructor()
	declare destructor()
	
	declare Sub Update()			'' This updates the player depending on its State
	declare sub Draw()				'' Draws the Player according to state
	
private:
	
	declare Sub UpdateIdle()			''\  These are the functions to be called by Update
	declare Sub UpdateWalking()			'' > Depending on the Player.State 
	declare Sub UpdateJumping()			''/ 
	
	x		as single				'' Position
	y		as single
	Dx		as single				'' Direction
	Dy		as Single
	Speed	as single				'' Horizontal Speed
	Wid		as integer				'' Width of the player
	Hei		as integer				
	CanJump as integer				'' If the player can Jump
	
	State	as E_STATE				'' State of the player
	
End Type

''*****************************************************************************
''
''*****************************************************************************
constructor Player()
	
	x = 32 * 2 
	y = 32 * 5 
	Speed = 2.5
	Wid = 24
	Hei = 48
	CanJump = false
	
	State = JUMPING
	
End Constructor

''*****************************************************************************
''
''*****************************************************************************
destructor Player()

End Destructor



''*****************************************************************************
'' Updates the player according to states
''*****************************************************************************
sub Player.Update()

	'' Check to see the Player State and
	'' Update accordingly
	
	'' In my DS game I used function pointers 
	'' But I have no idea how to get and call
	'' function pointers of class methods in FB
	'' so I used select case which is not too bad.
	Select Case State
		case IDLE:
			UpdateIdle()
		case WALKING:
			UpdateWalking()
		case JUMPING:
			UpdateJumping()
	End Select
    
    
End Sub

''*****************************************************************************
'' Called when player is not moving and standing on the ground
''*****************************************************************************
Sub Player.UpdateIdle()
	
	Dx = 0  '' Set speed to 0
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    EndIf
    
    '' We can jump while not moving so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump again while in the middle of Jumping
	    end if
    EndIf
    
	
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei + Dy > FLOOR_VALUE ) then    
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true		'' We hit floor so we can jump again
	End If
		
	
End Sub

''*****************************************************************************
'' Called when player is walking on the ground
''*****************************************************************************
Sub Player.UpdateWalking()
	
	'' Assume we are not moving so set
	'' State to be idle
	State = IDLE
	
	Dx = 0  '' Set speed to 0
	
	'' If we pressed left then we walk negatively
	'' and we set the State to WALKING since we moved
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
		State = WALKING
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    	State = WALKING
    EndIf
        
    '' We can jump while not moving so jump when we press space
    '' Then set the state to JUMPING
    if multikey(FB.SC_SPACE) then 
	    if( CanJump ) then
	    	State = JUMPING 		
	    	Dy = -JUMPHEIGHT		'' This makes us jump
	    	CanJump = false			'' We can't jump while Jumping
	    end if
    EndIf
    
	
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei + Dy > FLOOR_VALUE ) then    
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true		'' We hit floor so we can jump again
	End If
		
	
End Sub

''*****************************************************************************
'' Called when player is jumping
''*****************************************************************************
Sub Player.UpdateJumping()
	
	Dx = 0  '' Set speed to 0
	'' We can move left or right when jumping so...
	
	'' If we pressed left then we walk negatively
	if multikey(FB.SC_LEFT) then 
		Dx = -speed
	EndIf
	
	'' See comments above
    if multikey(FB.SC_RIGHT) then 
    	Dx = speed
    EndIf
    
	'' Make the player fall down with GRAVITY
	Dy += GRAVITY
	
	x += Dx		'' Move player
	y += Dy		'' Ditto
	
	'' y = 400 is our arbitrary floor so set Dy = 0
	'' and snap the y position to 400 - Hei
	If( y + Hei + Dy > FLOOR_VALUE ) then
		Dy = 0
		y = FLOOR_VALUE - Hei
		CanJump = true	'' We hit floor so we can jump again
		State = IDLE  '' And set the state to standing		
	End If
	
End Sub

''*****************************************************************************
'' Draws the player according to state
''*****************************************************************************
sub Player.Draw()

	Select Case State
		case IDLE:
			GL2D.BoxFilled( x, y, x + Wid, y + Hei, GL2D_RGBA(0,0,255,255) )
			GL2D.PrintScale(0, 0, 1, "State = IDLE" )
		case WALKING:
			GL2D.BoxFilled( x, y, x + Wid, y + Hei, GL2D_RGBA(255,0,0,255) )
			GL2D.PrintScale(0, 0, 1, "State = WALKING" )
		case JUMPING:
			GL2D.BoxFilled( x, y, x + Wid, y + Hei, GL2D_RGBA(0,255,0,255) )
			GL2D.PrintScale(0, 0, 1, "State = JUMPING" )
	End Select
		
		GL2D.PrintScale(0,  30, 1, "Box Is Colored differently depending on the Player's state.")    
		
		GL2D.PrintScale(0,  50, 1, "CONTROLS")    
		GL2D.PrintScale(0,  70, 1, "ARROWS = Move")    
		GL2D.PrintScale(0,  80, 1, " SPACE = Jump")    
	    
	
end sub


''*****************************************************************************
'' Our main sub
''*****************************************************************************
sub main()

	dim as integer Frame = 0
	 
	dim as Player Mario
	 
	GL2D.ScreenInit( SCREEN_WIDTH, SCREEN_HEIGHT )   ''Set up GL screen
	GL2D.VsyncON()
	
	
	do
		
		Frame += 1

		Mario.Update()     '' Do player movements
    
		GL2D.ClearScreen()
			
		GL2D.Begin2D()
			
			'' Draw floor
			GL2D.BoxFilled( 0, FLOOR_VALUE, SCREEN_WIDTH, SCREEN_HEIGHT, GL2D_RGBA(255,255,255,255) )
			
			'' Draw player
			Mario.Draw()
			
			
		GL2D.End2D()
		
		
		dim as integer FPS = GL2D.LimitFPS(60)
		
		sleep 1,1
		flip
		
	Loop until multikey(FB.SC_ESCAPE)

	GL2D.ShutDown()
	
End Sub

''*****************************************************************************
''*****************************************************************************


main()


end







