REM************************************************************************
REM                                                                       *
REM --- SAILING in QBASIC --------------- by Stephan Scholz, July 2012    *
REM     -----------------                 ----------------------------    *
REM                                                                       *
REM Sail round two four-buoy race courses in a 40 ft racing yacht and try *
REM to beat the AI opponent!                                              *
REM                                                                       *
REM LEFT-RIGHT CURSOR ARROWS: Rudder control.                             *
REM    UP-DOWN CURSOR ARROWS: Let out or pull in sail.                    *
REM                SPACE BAR: Deploy or stow SPINNAKER.                   *
REM                                                                       *
REM Detailed instructions are available from the main menu, and there is  *
REM in-game guidance by pressing the "h" key (help). Two different race   *
REM courses and five opponent ability levels can be selected.             *
REM                                                                       *
REM Your yacht's performance corresponds to the Soto-40 OD class racing   *
REM yacht in a 23-knot wind, for the following true wind angles:          *
REM                                                                       *
REM  40-50  degrees:     8   - 9    kt                                    *
REM  70-90  degrees:    10   - 11.4 kt                                    *
REM 120-150 degrees:    12.5 - 14.8 kt (Spinnaker up)                     *
REM 160-180 degrees:    13.5 - 11.3 kt (Spinnaker up)                     *
REM                                                                       *
REM Press "ESC" to QUIT at anytime.  NOTE: Caps Lock OFF.                 *
REM                                                                       *
REM************************************************************************

' Keyboard Capture Code...    by Justin Richards, edited
' for the tutorial by Lachie Dazdarian (July 1012)

CONST Key_Up& = 18432
CONST Key_Down& = 20480
CONST Key_Left& = 19200
CONST Key_Right& = 19712
CONST KEY_Esc& = 27
CONST Key_Space& = 32
CONST Key_Enter& = 13
CONST Key_M& = 109
CONST Key_H& = 104

SCREEN 12: CLS
REM ----------- screen size ----------
h = 480
w = 640

REM --------- Yacht dimensions -------
hull = 18: REM ------- hull size
boom = hull / 1.3
rudder = hull / 3.5
mastpos = hull / 2.2
hullcol = 2
flecha = 17: REM ----- body width
transom = hull / 2.7
transomangle = 130

REM ------------------ some constants
windspeed = 23
windangle = -90
pi = 3.14162
max = 60

REM ------------------ Displayed movement/accelleration:
speedscale = .1
REM ------------------ Turning speed:
turnscale = .1

REM ------------------ set up race type and difficulty
DIM opponent$(5)
DIM opponent(5)
DIM numlaps$(5)
DIM difficulty$(2)
GOSUB racevals
GOSUB menu

REM ------------------ wind instrument positions
wix = 244
wiy = 64

wixu = 396
wiyu = 64

REM ------------------ sailing characteristics
DIM se(180)
DIM spin(180)
DIM cd(360)
DIM rsf(180)

near = 30
buoyzone = 25

GOSUB Sailefficiency
GOSUB spinnaker
GOSUB resfactor
GOSUB Competitordata

REM ---------------------- waypoints
DIM waypx(4, 2)
DIM waypy(4, 2)
DIM waypxu(7, 2)
DIM waypyu(7, 2)
GOSUB waypoint

reentry:
REM --------- your lap counter
lapcount = 0
lastbuoy = 0

REM --------- AI yacht lap counter
lapcountu = 0
lastbuoyu = 0

REM --------------- starting position and variables
x = 620
y = 410
ve = 0
boomangle = 35
heading = 30
rudderangle = 45
spi = -1

REM ---------------- Competitor, initial variable values
xu = 590
yu = 450
veu = 0
headingu = 40
boomangleu = 35
rudderangleu = 0

wp = 1
wpu = 1

REM +++++++++++++++++++++++++++++ MAIN LOOP ++++++++++++++++++++++++++
REM ------------------------------------------------------------------

DO

    REM ----------------------- DETECT BUOY ------------------------------
    REM --------------- Buoyzones, Buoys and Waypoints -------------------
    REM -------------- Mark buoy to head for with red circle behind ------
    CIRCLE (waypx(wp, r), waypy(wp, r)), buoyzone, 4

    REM ----------- Extra waypoints for AI Yacht ----------------------
    REM ----------- Important for tacking in 2nd race and -------------
    REM ----------- Useful for steering adjustments in 1st race -------

    CIRCLE (waypxu(2, r), waypyu(2, r)), 3, 9, , , 1
    CIRCLE (waypxu(3, r), waypyu(3, r)), 3, 9, , , 1
    CIRCLE (waypxu(6, r), waypyu(6, r)), 3, 9, , , 1

    REM ----------- WAYPOINT CALCULATION ---------------------------------
    hdis = INT((x - waypx(wp, r)))
    vdis = INT((y - waypy(wp, r)))
    IF vdis <> 0 THEN prop = hdis / vdis
    atan = ATN(prop) / pi * 180

    REM -- Depending on the trigonometrical quadrant of your position ----
    REM -- in relation to the buoy you must head for, the arctangent -----
    REM -- of the horizontal distance divided by the vertical distance ---
    REM -- will give the bearing angle to the buoy -----------------------

    IF SGN(hdis) = -1 AND SGN(vdis) = 1 THEN bearing = -atan
    IF SGN(hdis) = 1 AND SGN(vdis) = 1 THEN bearing = 360 - atan
    IF SGN(hdis) = 1 AND SGN(vdis) = -1 THEN bearing = 180 - atan
    IF SGN(hdis) = -1 AND SGN(vdis) = -1 THEN bearing = 180 - atan

    REM ---------------------------- LAP COMPLETED ? ---------------------
    IF ABS(hdis) < buoyzone AND ABS(vdis) < buoyzone THEN
        SOUND 300, .3: SOUND 600, .3: SOUND 900, .3
        CIRCLE (waypx(wp, r), waypy(wp, r)), buoyzone, 0
        wp = wp + 1
        IF wp > 4 THEN: wp = 1: lastbuoy = 1
        IF lastbuoy = 1 AND wp = 2 THEN
            lapcount = lapcount + 1: lastbuoy = 0
            IF lapcount = laps THEN
                SOUND 400, 1.2
                SOUND 600, 1.2
                SOUND 800, 1.2
                SOUND 37, 1
                SOUND 1000, 3
                SOUND 37, 1
                SOUND 800, .7
                SOUND 37, 1
                SOUND 1000, 4
                IF lapcount > lapcountu THEN
                    LOCATE 13, 30: COLOR 12: PRINT "   Congratulations!   "
                    LOCATE 14, 30: COLOR 10: PRINT "You have won the race!"
                ELSE
                    LOCATE 16, 30: COLOR 12: PRINT "You have finished the race!"
                    LOCATE 17, 30: COLOR 12: PRINT "     A Good try!"
                END IF

                _DELAY 5
                LOCATE 19, 30: COLOR 14: PRINT "    SPACE to continue"

                DO
                    SpaceOn = _KEYDOWN(Key_Space&)
                LOOP UNTIL SpaceOn

                FOR lini = 13 TO 19: LOCATE lini, 30: PRINT STRING$(30, " "): NEXT lini
                GOSUB menu
                GOTO reentry
            END IF
        END IF
    END IF

    headi = heading - 90
    IF headi < 0 THEN headi = headi + 360
    IF headi > 360 THEN headi = headi - 360

    REM --------------------- TRUE WIND ----------------------------------
    REM ---------- Wind angle in relation to your heading ----------------
    relwind = (windangle - heading) + 180
    IF relwind < -180 THEN relwind = relwind + 360
    IF relwind > 360 THEN relwind = relwind - 360

    REM ---------------- WINDWARD SIDE RIGHT(+) OR LEFT(-) ---------------
    tk = SGN(relwind)

    REM ------------------- SPEED AND ANGLE OF APPARENT WIND -------------
    REM ----- angle in relation to your boat - varies with speed ---------
    Vay = windspeed * SIN(pi * relwind / 180)
    Vax = windspeed * COS(pi * relwind / 180) + ve
    Vapwind = SQR(Vax * Vax + Vay * Vay)
    Aapwind = (180 / pi) * ATN(Vay / Vax)
    IF ABS(relwind - Aapwind) > 179 THEN Aapwind = Aapwind + 180 * tk

    REM --------------------------- BOOM ANGLE ---------------------------
    REM ----- Boom colour changes with wind angle of incidence -----------
    REM ----- laminar flow, push only, or flapping in the wind -----------
    sil = boomangle * tk
    sail = sil
    difang = ABS(Aapwind) - ABS(sil)
    IF difang > 9 AND difang < 26 THEN boomcol = 11 ELSE boomcol = 12
    IF difang < 10 THEN boomcol = 8

    REM ------------- boom will trail if loose in the wind
    IF ABS(sil) >= ABS(Aapwind) THEN sail = Aapwind: difang = 0: boomcol = 8

    REM ------------------------ WIND FORCE VECTORS ----------------------
    REM -- Sail efficiency depends on apparent wind angle of -------------
    REM -- incidence on sail, and determines percentage of apparent ------
    REM -- wind force used for propulsion depending on boomangle ---------
    ins = INT(difang)
    sefi = se(ins)
    fsail = ABS(sil * pi / 180)

    REM ----------------------WATER AND AIR RESISTANCE -------------------
    REM --- Resistance factor read from resistance table for each --------
    REM --- sailing angle and multiplied to square of yacht's speed ------
    rsf = rsf(ABS(relwind))
    res = ve * ve * rsf

    REM --------------------- FINAL PROPULSION CALCULATION ---------------
    REM --- For raised spinnaker this force is read from the -------------
    REM --- spinnaker table for each sailing angle and added -------------
    REM --- to propulsion vector. Resistance is subtracted ---------------
    REM --- If wind is stolen then loss of 10% propulsion ----------------
    IF spi = 1 THEN spina = spin(ABS(relwind)) * 1.5 ELSE spina = 0
    prop = reduce * .0246 * sefi * SIN(fsail) * Vapwind - res + spina

    REM ------------- ACCELLERATE/DECELLERATE AS PER PROPULSION ----------
    REM ------------- Inertia makes for slower decelleration -------------
    IF ve < prop THEN ve = ve + .1
    IF ve > prop THEN ve = ve - .05

    REM ve = (ve - SGN(ve - prop) * .05)
    IF ve > max THEN ve = max
    IF ve < 0 THEN ve = 0

    REM ----------- SPINNAKER ALARM --------------------
    IF spi = 1 AND ABS(relwind) < 90 THEN SOUND 2000, .1

    REM ======================== DRAW SAILING YACHT ======================
    REM ---- hull line angles, boom and rudder positions and angles ------
    n = (heading + flecha) * pi / 180
    n1 = (heading - flecha) * pi / 180
    n2 = heading * pi / 180
    boomang = heading + sail
    n3 = boomang * pi / 180
    rudderang = heading + rudderangle
    n4 = rudderang * pi / 180
    n5 = (heading + transomangle) * pi / 180
    n6 = (heading - transomangle) * pi / 180

    REM ------ avoid screen flicker ---------------------
    WAIT &H3DA, 8
    REM slow down the programme for QB64
    _DELAY .036

    REM ------------ DRAW 4 COLOURED BUOYS --------------
    FOR ci = 1 TO 7 STEP 2
        CIRCLE (bx1(r), by1(r)), ci, 10, , , 1
        CIRCLE (bx2(r), by2(r)), ci, 14, , , 1
        CIRCLE (bx3(r), by3(r)), ci, 9, , , 1
        CIRCLE (bx4(r), by4(r)), ci, 12, , , 1
    NEXT ci

    REM ----------- ERASE HULL, BOOM AND RUDDER ---------
    LINE (x, y)-(x1, y1), 0
    LINE (x, y)-(x2, y2), 0
    LINE (x5, y5)-(x6, y6), 0
    LINE (x5, y5)-(x7, y7), 0
    LINE (x5, y5)-(x8, y8), 0
    LINE (x3, y3)-(x4, y4), 0

    REM ------- also for competitor ---------------------
    LINE (xu, yu)-(x1u, y1u), 0
    LINE (xu, yu)-(x2u, y2u), 0
    LINE (x5u, y5u)-(x6u, y6u), 0
    LINE (x5u, y5u)-(x7u, y7u), 0
    LINE (x5u, y5u)-(x8u, y8u), 0
    LINE (x3u, y3u)-(x4u, y4u), 0

    REM ----- ERASE WIND and bearing needles, and sheet indicator ------
    LINE (wix, wiy)-(xw, yw), 0
    LINE (wix, wiy)-(xaw, yaw), 0
    LINE (wix, wiy)-(xbe, ybe), 0
    LINE (wixu, wiyu)-(xwu, ywu), 0
    LINE (wixu, wiyu)-(xawu, yawu), 0
    LINE (wixu, wiyu)-(xbeu, ybeu), 0
    LINE (wix - 45, wiy + 56)-(wix + 45, wiy + 56), 0
    LINE (wix - 45, wiy + 57)-(wix + 45, wiy + 57), 0
    LINE (wix - 45, wiy + 58)-(wix + 45, wiy + 58), 0
    LINE (wix - 45, wiy + 59)-(wix + 45, wiy + 59), 0
    LINE (wix - 45, wiy + 60)-(wix + 45, wiy + 60), 0
    LINE (wix - 45, wiy + 61)-(wix + 45, wiy + 61), 0

    REM ------------------- KEEP IT ON SCREEN -----------
    IF x > w THEN x = w
    IF y > h THEN y = h
    IF x < 0 THEN x = 0
    IF y < 0 THEN y = 0

    REM -------- SPEED VECTORS ON-SCREEN MOVEMENT ----------------
    x = x - (ve * COS(n2)) * speedscale
    y = y - (ve * SIN(n2)) * speedscale

    REM ------ PIXEL COORDS AND ANGLE - HULL, BOOM, RUDDER -------
    x1 = x + hull * COS(n)
    y1 = y + hull * SIN(n)
    x2 = x + hull * COS(n1)
    y2 = y + hull * SIN(n1)
    x3 = x + mastpos * COS(n2)
    y3 = y + mastpos * SIN(n2)
    x4 = x3 + boom * COS(n3)
    y4 = y3 + boom * SIN(n3)
    x5 = x + hull * 1.2 * COS(n2)
    y5 = y + hull * 1.2 * SIN(n2)
    x6 = x5 + rudder * COS(n4)
    y6 = y5 + rudder * SIN(n4)
    x7 = x5 + transom * COS(n5)
    y7 = y5 + transom * SIN(n5)
    x8 = x5 + transom * COS(n6)
    y8 = y5 + transom * SIN(n6)

    REM ----------------  INSTRUMENT and needle coords -------------------
    wind = (Aapwind - 90) * pi / 180
    rwind = (relwind - 90) * pi / 180
    xw = wix + windspeed * 2 * COS(rwind)
    yw = wiy + windspeed * 2 * SIN(rwind)
    xaw = wix + windspeed * 2 * COS(wind)
    yaw = wiy + windspeed * 2 * SIN(wind)

    windu = (Aapwindu - 90) * pi / 180
    rwindu = (relwindu - 90) * pi / 180
    xwu = wixu + windspeed * 2 * COS(rwindu)
    ywu = wiyu + windspeed * 2 * SIN(rwindu)
    xawu = wixu + windspeed * 2 * COS(windu)
    yawu = wiyu + windspeed * 2 * SIN(windu)

    beari = (bearing - heading) * pi / 180
    xbe = wix + windspeed * 2 * COS(beari)
    ybe = wiy + windspeed * 2 * SIN(beari)

    beariu = (bearingu - headingu) * pi / 180
    xbeu = wixu + windspeed * 2 * COS(beariu)
    ybeu = wiyu + windspeed * 2 * SIN(beariu)

    REM ------------------ DRAW HULL, BOOM AND RUDDER --------------------
    LINE (x, y)-(x1, y1), hullcol
    LINE (x, y)-(x2, y2), hullcol
    LINE (x5, y5)-(x6, y6), 10
    LINE (x5, y5)-(x7, y7), hullcol
    LINE (x5, y5)-(x8, y8), hullcol
    LINE (x3, y3)-(x4, y4), boomcol

    REM ======================= AI OPPONENT YACHT ========================
    REM ==================================================================
    headu = headingu - 90
    IF headu < 0 THEN headu = headu + 360

    REM -------------- DETECT BUOY ---------------------------------------
    IF counter > 0 THEN counter = counter - 1
    IF counter = 1 THEN
        FOR lin = 13 TO 14
            LOCATE lin, 35: PRINT STRING$(20, " ")
        NEXT lin
    END IF
    IF ABS(xu - waypxu(wpu, r)) < buoyzone AND ABS(yu - waypyu(wpu, r)) < buoyzone THEN
        wpu = wpu + 1
        IF wpu > 7 THEN lastbuoyu = 1: wpu = 1
        IF lastbuoyu = 1 AND wpu = 2 THEN lapcountu = lapcountu + 1: lastbuoyu = 0
        check = 0: look = 0
        IF lapcountu = laps THEN
            IF lapcountu > lapcount AND message = 0 THEN
                message = 1
                LOCATE 13, 35: COLOR 16 - oppvel: PRINT opponent$(oppvel)
                LOCATE 14, 35: COLOR 2: PRINT "has won the race!"
                counter = 150
                SOUND 1000, 1.2
                SOUND 800, 1.2
                SOUND 1000, 1.2
                SOUND 37, 1
                SOUND 600, 3
                SOUND 37, 1
                SOUND 400, .7
                SOUND 37, 1
                SOUND 600, 4
            END IF
        END IF
    END IF

    REM --- DETERMINE HEADING ---- WAYPOINT BEARING CALCULATION --------
    REM --- Depending on trigonometrical quadrant of AI yacht's --------
    REM --- position in relation to waypoint, acrtangent calculation ---
    REM --- of horizontal divided by vertical distance is corrected ----
    REM --- to give necessary bearing angle. Angle is calculated at ----
    REM --- intervals to allow time for AI tacking.
    hdisu = xu - waypxu(wpu, r)
    vdisu = yu - waypyu(wpu, r)
    IF vdisu <> 0 THEN propu = hdisu / vdisu
    atanu = ATN(propu) / pi * 180

    check = check - 1
    IF check < 1 THEN
        check = 45
        IF SGN(hdisu) = -1 AND SGN(vdisu) = 1 THEN bearingu = -atanu
        IF SGN(hdisu) = 1 AND SGN(vdisu) = 1 THEN bearingu = 360 - atanu
        IF SGN(hdisu) = 1 AND SGN(vdisu) = -1 THEN bearingu = 180 - atanu
        IF SGN(hdisu) = -1 AND SGN(vdisu) = -1 THEN bearingu = 180 - atanu
    END IF

    difu = ABS(headu - bearingu)
    IF difu < 12 AND ABS(relwindu) > 40 THEN GOTO fiu

    REM ---- Magnitude of steering angle set according ---------------
    REM ---- to difference between heading and bearing ---------------
    buoyu = INT(difu)
    IF buoyu > 60 THEN buoyu = 60

    REM -------- decide which way to turn, depending on difference between
    REM -------- current heading and bearing to buoy, including time counter
    REM -------- to allow for tacking.
    look = look - 1
    IF look < 1 THEN
        IF bearingu < headu THEN
            look = 25
            IF difu > 180 THEN rudderangleu = -buoyu ELSE rudderangleu = buoyu
        END IF

        IF bearingu > headu THEN
            look = 25
            IF difu > 180 THEN rudderangleu = buoyu ELSE rudderangleu = -buoyu
        END IF
    END IF
    fiu:

    REM ------------------ SPEED AND ANGLE OF APPARENT WIND AI --------------
    REM ------------ This is only for aI yacht instrument display -----------
    tku = SGN(relwindu)
    Vayu = windspeed * SIN(pi * relwindu / 180)
    Vaxu = windspeed * COS(pi * relwindu / 180) + veu
    Vapwindu = SQR(Vaxu * Vaxu + Vayu * Vayu)
    Aapwindu = (180 / pi) * ATN(Vayu / Vaxu)
    IF ABS(relwindu - Aapwindu) > 179 THEN Aapwindu = Aapwindu + 180 * tku

    REM -------------- AI YACHT SPEED ---------------------------------------
    REM ------ Matrix cd() gives wind speed at given angle 1-360 ------------
    REM ------ Matrix Oppspeed() gives 6 different opponent attitudes -------
    REM ------ If wind is stolen reduceu <1 ----------------------------------
    veul = cd(headu) * windspeed * oppspeed(oppvel) * reduceu

    REM ------------- Decelleration slower to aid slow opponent AI tacking --
    IF veul < veu THEN veu = veu - speedscale / 1.7
    IF veul > veu THEN veu = veu + speedscale

    REM ----------------- AI YACHT BOOM ANGLE AI ---------------------
    relwindu = (windangle - headingu) + 180
    IF relwindu < -180 THEN relwindu = relwindu + 360
    IF relwindu > 360 THEN relwindu = relwindu - 360
    tku = SGN(relwindu)
    silu = boomangleu * tku
    sailu = silu
    difangu = ABS(relwindu) - ABS(silu)
    IF difangu > 9 AND difangu < 26 THEN boomcolu = 11 ELSE boomcolu = 12

    REM -------------- BOOM LOOSE IN THE WIND ---------------------------
    IF ABS(silu) > ABS(relwindu) THEN sailu = relwindu: difangu = 0: boomcolu = 8
    IF boomcolu = 12 AND headingu > 180 AND boomangleu < 75 THEN boomangleu = boomangleu + 1
    IF boomcolu = 12 AND headingu < 180 AND boomangleu > 20 THEN boomangleu = boomangleu - 1


    REM -------------------- AI YACHT AI TURNING -----------------
    IF rudderangleu < 0 THEN headingu = headingu + .2 + (turnscale * veu): rudderangleu = rudderangleu + 2.5
    IF rudderangleu > 0 THEN headingu = headingu - .2 - (turnscale * veu): rudderangleu = rudderangleu - 2.5
    IF headingu < 1 THEN headingu = 360
    IF headingu > 360 THEN headingu = 1

    REM -------- HULL LINE ANGLES, BOOM AND RUDDER POSITIONS AND ANGLES --------
    nu = (headingu + flecha) * pi / 180
    n1u = (headingu - flecha) * pi / 180
    n2u = headingu * pi / 180
    boomangu = headingu + sailu
    n3u = boomangu * pi / 180
    rudderangu = headingu + rudderangleu
    n4u = rudderangu * pi / 180
    n5u = (headingu + transomangle) * pi / 180
    n6u = (headingu - transomangle) * pi / 180

    REM ------------------- KEEP IT ON SCREEN ----------------------------------
    IF xu > w THEN xu = w
    IF yu > h THEN yu = h
    IF xu < 0 THEN xu = 0
    IF yu < 0 THEN yu = 0

    REM ---------------- CALCULATE SPEED VECTORS ----------------
    xu = xu - (veu * COS(n2u)) * speedscale
    yu = yu - (veu * SIN(n2u)) * speedscale

    REM ------ PIXEL COORDS AND ANGLE - HULL, BOOM, RUDDER ------
    x1u = xu + hull * COS(nu)
    y1u = yu + hull * SIN(nu)
    x2u = xu + hull * COS(n1u)
    y2u = yu + hull * SIN(n1u)
    x3u = xu + mastpos * COS(n2u)
    y3u = yu + mastpos * SIN(n2u)
    x4u = x3u + boom * COS(n3u)
    y4u = y3u + boom * SIN(n3u)
    x5u = xu + hull * 1.2 * COS(n2u)
    y5u = yu + hull * 1.2 * SIN(n2u)
    x6u = x5u + rudder * COS(n4u)
    y6u = y5u + rudder * SIN(n4u)
    x7u = x5u + transom * COS(n5u)
    y7u = y5u + transom * SIN(n5u)
    x8u = x5u + transom * COS(n6u)
    y8u = y5u + transom * SIN(n6u)

    REM ------------- DRAW HULL, BOOM AND RUDDER ----------------
    LINE (xu, yu)-(x1u, y1u), hullcolu
    LINE (xu, yu)-(x2u, y2u), hullcolu
    LINE (x5u, y5u)-(x6u, y6u), hullcolu - 8
    LINE (x5u, y5u)-(x7u, y7u), hullcolu
    LINE (x5u, y5u)-(x8u, y8u), hullcolu
    LINE (x3u, y3u)-(x4u, y4u), boomcolu


    REM -----------KEYBOARD READING -- CONTROL KEYS ---------------------

    ' Keyboard Capture Code...    by Justin Richards, edited
    ' for the tutorial by Lachie Dazdarian (July, 2012)
    ' _KEYDOWN checks our keyboard capture constants
    ' and returns -1 if the key is being pressed.

    UpOn = _KEYDOWN(Key_Up&) '          let sail out
    DownOn = _KEYDOWN(Key_Down&) '      pull sail in
    LeftOn = _KEYDOWN(Key_Left&) '      left rudder
    RightOn = _KEYDOWN(Key_Right&) '    right rudder
    SpaceOn = _KEYDOWN(Key_Space&) '    toggle spinnaker up/down
    HelpOn = _KEYDOWN(Key_H&) '         in game Help
    MenuOn = _KEYDOWN(Key_M&) '         Menu
    EscOn = _KEYDOWN(KEY_Esc&) '        Quit

    IF LeftOn THEN IF rudderangle < 60 THEN rudderangle = rudderangle + 5

    IF RightOn THEN IF rudderangle > -60 THEN rudderangle = rudderangle - 5

    IF UpOn THEN
        IF boomangle < 90 THEN boomangle = boomangle + 1
        IF boomangle < 30 THEN boomangle = boomangle + 1
    END IF

    IF DownOn THEN
        IF boomangle > 0 THEN boomangle = boomangle - 1
        IF boomangle > 60 THEN boomangle = boomangle - 1
    END IF

    IF SpaceOn THEN
        spi = -spi
        IF SGN(spi) = 1 THEN SOUND 600, 1: SOUND 800, 1: SOUND 1000, 1:
        IF SGN(spi) = -1 THEN SOUND 700, 1: SOUND 500, 1: SOUND 300, 1
    END IF

    IF MenuOn THEN
        GOSUB menu
        GOTO reentry
    END IF

    IF HelpOn THEN GOSUB help

    IF EscOn THEN END

    REM ----------------------- RUDDER EFFECTIVITY ----------------------
    REM ---- Depends on speed and includes return back to neutral -------
    REM ---- Zero speed turns are possible - as if using an oar ---------
    IF rudderangle < 0 THEN heading = heading + .2 + (turnscale * ve): rudderangle = rudderangle + 2.5
    IF rudderangle > 0 THEN heading = heading - .2 - (turnscale * ve): rudderangle = rudderangle - 2.5
    IF heading < 0 THEN heading = heading + 360
    IF heading > 360 THEN heading = heading - 360

    REM ---------------- COLLISION + STEALING WIND DETECTION -----------------
    detx = ABS(INT(x3 - x3u))
    dety = INT(y3 - y3u)
    detya = ABS(dety)
    reduce = 1
    reduceu = 1

    REM LOCATE 10, 10: PRINT detx, detya

    IF detya < 30 AND detx < detya / 2 THEN
        redu = (detx + detya) / 100
        IF SGN(dety) = 1 THEN reduce = redu: SOUND 300, .1
        IF SGN(dety) = -1 THEN reduceu = 1 - redu: REM SOUND 900, .1
    END IF

    IF detya < 9 AND detx < 9 THEN
        SOUND 100, .1
        heading = heading + (RND * 11) - 5
        headingu = headingu + (RND * 11) - 5
    END IF

    REM-------- WIND INSTRUMENT ---------------------------------------------
    velo = INT(ve * 10) / 10
    COLOR 10: LOCATE 1, 14: PRINT "`Esmeralda'"
    COLOR 2: LOCATE 2, 14: PRINT "Heading"; INT(headi); " "
    LOCATE 3, 17: PRINT "Lap"; lapcount
    COLOR 10: LOCATE 5, 29: PRINT velo; "  "
    LOCATE 6, 29: PRINT "Spi ";
    IF spi = 1 THEN PRINT CHR$(24)
    IF spi = -1 THEN PRINT CHR$(25)
    COLOR 7: LOCATE 7, 29: PRINT INT(ABS(relwind))

    CIRCLE (wix, wiy), 52, 10, , , 1
    LINE (wix, wiy)-(xw, yw), 15
    LINE (wix, wiy)-(xaw, yaw), 11
    LINE (wix, wiy)-(xbe, ybe), 12
    LINE (wix - 15, wiy)-(wix + 21, wiy + 13), 7, B
    LINE (wix - 46, wiy + 55)-(wix + 46, wiy + 62), 7, B
    LINE (wix - 45, wiy + 56)-(wix - 45 + boomangle, wiy + 56), 6
    LINE (wix - 45, wiy + 57)-(wix - 45 + boomangle, wiy + 57), 6
    LINE (wix - 45, wiy + 58)-(wix - 45 + boomangle, wiy + 58), 6
    LINE (wix - 45, wiy + 59)-(wix - 45 + ABS(sail), wiy + 59), boomcol
    LINE (wix - 45, wiy + 60)-(wix - 45 + ABS(sail), wiy + 60), boomcol
    LINE (wix - 45, wiy + 61)-(wix - 45 + ABS(sail), wiy + 61), boomcol

    REM------------------- AI Yacht instrument---------------------------------
    COLOR 16 - oppvel: LOCATE 1, 55: PRINT "`"; opponent$(oppvel); "'"
    COLOR 8 - oppvel: LOCATE 2, 58: PRINT "Heading"; INT(headu); " "
    LOCATE 3, 60: PRINT "Lap"; lapcountu
    COLOR 8 - oppvel: LOCATE 5, 48: PRINT INT(veu * 10) / 10; "  "
    COLOR 8 - oppvel: LOCATE 6, 48: PRINT "Spi ";
    IF ABS(relwindu) > 95 THEN PRINT CHR$(24) ELSE PRINT CHR$(25)
    COLOR 7: LOCATE 7, 48: PRINT INT(ABS(relwindu))

    CIRCLE (wixu, wiyu), 52, 16 - oppvel, , , 1
    LINE (wixu, wiyu)-(xwu, ywu), 15
    LINE (wixu, wiyu)-(xawu, yawu), 11
    LINE (wixu, wiyu)-(xbeu, ybeu), 12
    LINE (wixu - 15, wiyu)-(wixu + 21, wiyu + 13), 16 - oppvel, B

LOOP

REM --------------------- SAIL EFFICIENCY DATA ----------------------------
Sailefficiency:
RESTORE incidence
FOR n = 1 TO 180
    READ a
    se(n) = a
NEXT n
incidence:
DATA 0,0,0,0,0,0,0,0,0,0
DATA 80,81,83,85,87,89,91,93,95,97
DATA 98,99,100,99,98,97,70,70,70,70
DATA 70,70,70,70,70,70,70,70,70,70
DATA 70,70,70,70,70,70,71,71,72,72
DATA 73,73,74,74,75,75,75,75,75,75
DATA 75,75,75,75,75,75,75,75,75,75
DATA 75,75,75,75,75,75,75,75,75,75
DATA 75,76,77,78,79,80,80,80,80,80
DATA 79,78,77,76,75,74,73,72,71,70
DATA 69,68,67,66,65,64,63,62,61,60
DATA 59,58,57,56,55,54,53,52,51,50
DATA 49,48,47,46,45,44,43,42,41,40
DATA 39,38,37,36,35,34,33,32,31,30
DATA 29,28,27,26,25,24,23,22,21,20
DATA 19,18,17,16,15,14,13,12,11,10
DATA 9,8,7,6,5,4,3,2,1,0
DATA 0,0,0,0,0,0,0,0,0,0,0
RETURN

REM -------------------- SPINNAKER DATA ----------------------
spinnaker:
REM ------- useless angles:
FOR n = 1 TO 89
    spin(n) = -100
NEXT n

REM ------- useful angles:
RESTORE spi
FOR n = 90 TO 180
    READ a
    spin(n) = a
NEXT n
spi:
DATA 1,2,3,4,5,6,7,8,9,10
DATA 11,12,13,14,15,16,17,18,19,20
DATA 22,24,26,28,30,32,34,36,38,40
DATA 42,44,46,48,50,52,54,56,58,60
DATA 62,65,68,71,74,75,78,81,83,85
DATA 88,87,86,85,84,83,82,81,80,79
DATA 78,76,74,72,70,68,66,64,62,60
DATA 58,56,54,52,50,48,46,44,42,40
DATA 38,36,34,32,30,28,26,25,24,23,23

RETURN

REM ------------------- RESISTANCE FACTOR DATA --------------
REM ---- Wind resistance increases for close-hauled angles, and is nil
REM ---- below 30 degrees relative wind. Water resistance increases for
REM ---- running angles due to pitching. For each relative wind angle,
REM ---- value is read from the matrix and multiplied by the value of
REM ---- the angle in degrees, giving resistance factor for that angle.

resfactor:
RESTORE Resistance
FOR n = 1 TO 180
    READ a
    rsf(n) = a
NEXT n
Resistance:
DATA 100,100,100,100,100,100,100,100,100,100
DATA 100,100,100,100,100,100,100,100,100,100
DATA 100,100,100,100,100,100,100,100,100,98
DATA 98,99,99,100,100,101,102,103,103,104
DATA 104,104,104,104,104,105,105,105,105,105
DATA 106,106,107,107,108,108,109,109,110,110
DATA 109,109,108,108,107,107,106,106,105,105
DATA 104,103,102,101,100,98,96,94,92,90
DATA 88,86,84,84,84,84,83,81,79,77
DATA 78,80,83,86,89,92,95,98,101,104
DATA 106,108,110,112,114,116,118,120,122,124
DATA 126,129,132,135,138,140,142,144,146,148
DATA 150,152,154,156,158,160,162,164,166,168
DATA 167,167,166,166,165,165,164,164,163,162
DATA 159,156,152,148,143,138,133,128,124,120
DATA 117,114,111,108,105,102,100,98,96,94
DATA 92,90,88,86,84,82,80,78,77,76
DATA 75,71,68,65,64,63,62,61,61,60

FOR n = 1 TO 180
    rsf(n) = rsf(n) / 30000 * n
NEXT n

RETURN

REM ------------- AI YACHT PERFORMANCE DATA ------------
REM ------------ Speed for each relative wind angle -----------
REM --------- Running angles include spinnaker values ---------
Competitordata:
RESTORE autos
FOR n = 10 TO 180
    READ a
    cd(n) = a
    cd(360 - n) = a
NEXT n
autos:
DATA 20,22,24,26,28,30,32,34,36,38
DATA 40,42,44,46,48,50,52,54,56,58
DATA 60,61,62,63,64,65,66,67,68,69
DATA 70,70,71,71,72,72,73,73,74,74
DATA 75,75,76,76,77,77,78,78,79,79
DATA 80,80,81,81,82,82,83,83,84,84
DATA 85,85,86,86,87,87,88,88,89,89
DATA 90,90,91,91,92,92,93,93,94,94
DATA 95,95,96,96,97,97,98,98,99,99
DATA 100,100,101,102,102,103,103,104,104,105
DATA 105,106,106,107,107,108,108,109,109,110
DATA 110,111,111,112,112,113,113,114,114,115
DATA 115,116,116,117,117,118,118,119,119,120
DATA 120,121,121,122,122,123,123,124,124,125
DATA 125,124,124,123,123,122,122,121,121,120
DATA 120,119,118,117,116,115,114,113,112,111
DATA 110,109,108,107,106,105,104,103,102,101,100

RETURN

REM ---------- BUOY POSITIONS for two race courses ---------
waypoint:
bx1(1) = 540
by1(1) = 390
bx2(1) = 90
by2(1) = 90
bx3(1) = 180
by3(1) = 390
bx4(1) = 560
by4(1) = 110

bx1(2) = 540
by1(2) = 390
bx2(2) = 540
by2(2) = 90
bx3(2) = 80
by3(2) = 370
bx4(2) = 500
by4(2) = 425

REM -------- Waypoint positions adjusted at points near buoys
REM -------- For your yacht, for two race courses
waypx(1, 1) = bx1(1) + near - 10
waypy(1, 1) = by1(1) + near - 10
waypx(2, 1) = bx2(1) - near + 10
waypy(2, 1) = by2(1) - near + 10
waypx(3, 1) = bx3(1) - near + 10
waypy(3, 1) = by3(1) + near - 10
waypx(4, 1) = bx4(1) + near - 10
waypy(4, 1) = by4(1) - near + 10

waypx(1, 2) = bx1(2) + near
waypy(1, 2) = by1(2)
waypx(2, 2) = bx2(2)
waypy(2, 2) = by2(2) - near
waypx(3, 2) = bx3(2) - near
waypy(3, 2) = by3(2) - 8
waypx(4, 2) = bx4(2)
waypy(4, 2) = by4(2) + near

REM ------- Buoy waypoints for AI Yacht. 1,4,5,7 are same as
REM ------- 1,2,3,4 for player Yacht. Buoys 2,3 and 6 only set
REM ------- route so AI yacht can tack with enough speed.
waypxu(1, 1) = bx1(1) + near - 10
waypyu(1, 1) = by1(1) + near - 10
waypxu(2, 1) = bx2(1) + 300
waypyu(2, 1) = by2(1) + 200
waypxu(3, 1) = bx2(1) + 100
waypyu(3, 1) = by2(1) + 10
waypxu(4, 1) = bx2(1) - near + 10
waypyu(4, 1) = by2(1) - near + 10
waypxu(5, 1) = bx3(1) - near + 10
waypyu(5, 1) = by3(1) + near - 10
waypxu(6, 1) = bx4(1) - 100
waypyu(6, 1) = by4(1) + 70
waypxu(7, 1) = bx4(1) + near - 10
waypyu(7, 1) = by4(1) - near + 10

waypxu(1, 2) = bx1(2) + near
waypyu(1, 2) = by1(2)
waypxu(2, 2) = bx2(2) - 100
waypyu(2, 2) = by2(2) + 190
waypxu(3, 2) = bx2(2) + 70
waypyu(3, 2) = by2(2) + 70
waypxu(4, 2) = bx2(2)
waypyu(4, 2) = by2(2) - near
waypxu(5, 2) = bx3(2) - near
waypyu(5, 2) = by3(2) - 8
waypxu(6, 2) = bx3(2) + 100
waypyu(6, 2) = by4(2)
waypxu(7, 2) = bx4(2)
waypyu(7, 2) = by4(2) + near

menu:
message = 0
CLS
COLOR 7: LOCATE 4, 31: PRINT " Yacht";: COLOR 10: PRINT " `Esmeralda'"
COLOR 7: LOCATE 5, 31: PRINT " Soto-40 OD Class"

LINE (221, 41)-(400, 84), 9, B
LINE (222, 42)-(399, 83), 2, B

COLOR 11: LOCATE 12, 32: PRINT " Sailing Options"
LINE (221, 172)-(400, 194), 9, B
LINE (222, 173)-(399, 193), 2, B
LINE (120, 90)-(500, 166), 9, B
LINE (121, 91)-(499, 165), 2, B
LINE (120, 200)-(500, 390), 9, B
LINE (121, 201)-(499, 389), 2, B
LINE (100, 34)-(520, 411), 2, B
LINE (101, 35)-(519, 410), 9, B
PAINT (104, 53), 1, 9

LOCATE 7, 34: COLOR 12: PRINT "Control Keys"
LOCATE 8, 17: COLOR 14: PRINT "Left/Right Cursor:";: COLOR 2: PRINT " Steering (Rudder)"
LOCATE 9, 17: COLOR 14: PRINT "   Up/down Cursor:";: COLOR 2: PRINT " Let Out / Pull In Sailboom"
LOCATE 10, 17: COLOR 14: PRINT "        SPACE BAR:";
COLOR 2: PRINT " Deploy or Stow Spinnaker"
LOCATE 14, 17: COLOR 9: PRINT " Use";: COLOR 14: PRINT " up/down";: COLOR 9: PRINT " arrows to";: COLOR 14: PRINT " choose";: COLOR 9: PRINT " an option, and"
LOCATE 15, 17: COLOR 9: PRINT " then press";: COLOR 11: PRINT " ENTER";: COLOR 9: PRINT " to";: COLOR 11: PRINT " select";: COLOR 9: PRINT " this option."
COLOR 7
LOCATE 18, 19: PRINT " - See detailed sailing instructions"
LOCATE 19, 19: PRINT " - Type of race: ";: COLOR 2: PRINT difficulty$(r)
COLOR 7: LOCATE 20, 19: PRINT " - Opponent's attitude: ";: COLOR 16 - oppvel: PRINT opponent$(oppvel)
COLOR 7: LOCATE 21, 19: PRINT " - The race has ";: COLOR 2: PRINT numlap$(laps);: COLOR 7: PRINT " lap"
LOCATE 22, 17: COLOR 12: PRINT "                During the game:"
LOCATE 23, 17: COLOR 6: PRINT "Press";: COLOR 14: PRINT " H ";: COLOR 6: PRINT "for ";
COLOR 3: PRINT "in-game help";: COLOR 6: PRINT ", and";: COLOR 14: PRINT " M ";: COLOR 6: PRINT "for this";
COLOR 3: PRINT " menu";: COLOR 6: PRINT "."
LOCATE 24, 17: COLOR 6: PRINT "Press";: COLOR 14: PRINT " ESC ";: COLOR 6: PRINT "anytime to";: COLOR 3: PRINT " Quit.";: COLOR 4: PRINT " >> Capslock OFF <<"
COLOR 14

REM ---------- Read Keyboard ----------------------
DO
    COLOR 14: LOCATE curs, 17: PRINT ">>"
    LOCATE curs, 61: PRINT "<<"

    DO
    LOOP UNTIL INKEY$ <> ""

    LOCATE curs, 17: PRINT "  "
    LOCATE curs, 61: PRINT "  "

    ' Keyboard Capture Code...    by Justin Richards, edited
    ' for the tutorial by Lachie Dazdarian (July, 2012)

    UpOn = _KEYDOWN(Key_Up&)
    DownOn = _KEYDOWN(Key_Down&)
    LeftOn = _KEYDOWN(Key_Left&)
    RightOn = _KEYDOWN(Key_Right&)
    EnterOn = _KEYDOWN(Key_Enter&)
    EscOn = _KEYDOWN(KEY_Esc&)

    IF EscOn THEN END

    IF EnterOn THEN
        IF difcu > 1 THEN SOUND 1200, .5: EXIT DO ELSE GOSUB instructions: GOTO menu
    END IF

    IF UpOn THEN IF curs > 18 THEN curs = curs - 1: SOUND 500, .5

    IF DownOn THEN IF curs < 21 THEN curs = curs + 1: SOUND 500, .5

    IF RightOn THEN
        SOUND 2000, .4
        IF curs = 19 AND r < 2 THEN r = r + 1
        IF curs = 20 AND oppvel < 5 THEN oppvel = oppvel + 1
        IF curs = 21 AND laps < 5 THEN laps = laps + 1
    END IF

    IF LeftOn THEN
        SOUND 1400, .4
        IF curs = 19 AND r > 1 THEN r = r - 1
        IF curs = 20 AND oppvel > 1 THEN oppvel = oppvel - 1
        IF curs = 21 AND laps > 1 THEN laps = laps - 1
    END IF

    difcu = curs - 17
    hullcolu = 16 - oppvel
    LOCATE 19, 36: COLOR 2: PRINT difficulty$(r)
    LOCATE 20, 43: COLOR 16 - oppvel: PRINT opponent$(oppvel)
    LOCATE 21, 35: COLOR 2: PRINT numlap$(laps);: COLOR 7: PRINT " laps  "
    IF laps = 1 THEN COLOR 4: LOCATE 21, 42: PRINT " "
    LOCATE 15, 18: COLOR 9

    IF difcu = 1 THEN
        PRINT "then press";: COLOR 11: PRINT " ENTER";: COLOR 9: PRINT " to";
        COLOR 11: PRINT " select";
        COLOR 9: PRINT " this option.     "
        LOCATE 16, 18: PRINT STRING$(45, " ")

    ELSE
        PRINT "use";: COLOR 12: PRINT " left/right";: COLOR 9: PRINT " arrows to";
        COLOR 12: PRINT " change";: COLOR 9: PRINT " this option."
        LOCATE 16, 18: PRINT "Press";: COLOR 11: PRINT " ENTER ";: COLOR 9: PRINT "to";
        COLOR 11: PRINT " sail";: COLOR 9: PRINT " with these options."

    END IF

LOOP
CLS
RETURN

instructions:
CLS
LINE (10, 6)-(627, 457), 12, B
LINE (11, 7)-(626, 456), 14, B
SOUND 500, .8
SOUND 800, .8
LOCATE 2, 28: COLOR 12: PRINT " General Information                            1"
LOCATE 3, 5: COLOR 7: PRINT "Your yacht is the";: COLOR 2: PRINT " green ";
COLOR 7: PRINT "one, and your opponent's is another colour. ";: COLOR 7: PRINT "On"
LOCATE 4, 5: PRINT "the top left of the screen is your wind instrument. It shows the wind"
LOCATE 5, 5: PRINT "direction, your speed and the";: COLOR 12: PRINT " route ";: COLOR 7: PRINT "to follow. The";: COLOR 15: PRINT " white needle ";: COLOR 7: PRINT "indicates"
LOCATE 6, 5: PRINT "the";: COLOR 15: PRINT " true wind";: COLOR 7: PRINT " in relation to your yacht, and the ";: COLOR 11: PRINT "light blue needle";: COLOR 7: PRINT " is the"
LOCATE 7, 5: COLOR 11: PRINT "apparent wind,";: COLOR 7: PRINT " which moves forward as you gain speed. The";: COLOR 15: PRINT " true wind";: COLOR 7: PRINT " is"
LOCATE 8, 5: PRINT "also given in numbers. On the side, your";: COLOR 2: PRINT " heading ";: COLOR 7: PRINT "appears in degrees North."
LOCATE 9, 5: PRINT "The";: COLOR 12: PRINT " red needle";: COLOR 7: PRINT " is the";: COLOR 12: PRINT " waypoint";
COLOR 7: PRINT " to the next buoy. The";: COLOR 11: PRINT " laps ";: COLOR 7: PRINT "that you have"
LOCATE 10, 5: COLOR 7: PRINT "completed, and the current state of the";: COLOR 14: PRINT " spinnaker";: COLOR 7: PRINT " are also shown. Below, a"
LOCATE 11, 5: PRINT "dual bar indicates the";: COLOR 6: PRINT " sheet length ";: COLOR 7: PRINT "and the sail position. The difference"
LOCATE 12, 5: PRINT "shows the ";: COLOR 6: PRINT "loose rope ";: COLOR 7: PRINT "to be pulled in when the sail";: COLOR 8: PRINT " flaps in the wind."
LOCATE 14, 28: COLOR 12: PRINT " Sailing Instructions "
LOCATE 15, 5: COLOR 9: PRINT "The wind comes from the north, and depending on your yacht's heading,"
LOCATE 16, 5: PRINT "your sail must be adjusted for the best speed. The sailboom colour will"
LOCATE 17, 5: PRINT "show different conditions:";: COLOR 8: PRINT " GREY";:
COLOR 9: PRINT " when the sail is";: COLOR 8: PRINT " flapping in the wind";
LOCATE 18, 5: COLOR 9: PRINT "and giving no push,";: COLOR 12: PRINT " ORANGE ";
COLOR 9: PRINT "when it is getting the wind";: COLOR 12: PRINT " only from behind,"
LOCATE 19, 5: COLOR 9: PRINT "and ";: COLOR 11: PRINT "LIGHT-BLUE ";
COLOR 9: PRINT "when ";: COLOR 11: PRINT "laminar flow ";: COLOR 9: PRINT " greatly increases sail efficiency at"
LOCATE 20, 5: PRINT "angles of incidence of wind on sail of 10 to 25 degrees. This is needed"
LOCATE 21, 5: PRINT "for sailing at upwind angles of 30-45 degrees to the wind";: COLOR 5: PRINT " (close hauled),"
LOCATE 22, 5: COLOR 9: PRINT "and effective with with sidewind";: COLOR 5: PRINT " (reaching),";
COLOR 9: PRINT " or rear sidewind";: COLOR 5: PRINT " (running)."
COLOR 3
LOCATE 23, 5: PRINT "Speeds correspond to the Soto-40 OD class racing yacht in a 23-knot wind:"
COLOR 6
LOCATE 24, 5: PRINT " 40-50  deg. :   8  - 9   kt"
LOCATE 25, 5: PRINT " 70-90  deg. :  10  -11.4 kt"
LOCATE 26, 5: PRINT "120-150 deg. :  12.5-14.8 kt (Spinnaker up)"
LOCATE 27, 5: PRINT "160-180 deg. :  13.5-11.3 kt (Spinnaker up)"
LOCATE 28, 62: COLOR 2: PRINT "Any key: Page 2"
pausa$ = INPUT$(1)
CLS

LINE (10, 6)-(627, 457), 12, B
LINE (11, 7)-(626, 456), 14, B
SOUND 500, .8
SOUND 800, .8
LOCATE 2, 33: COLOR 12: PRINT " Control Keys                              2"
COLOR 9
LOCATE 3, 5: PRINT "To let out or pull in the sail, use the ";: COLOR 14: PRINT "UP/DOWN arrows.";: COLOR 9: PRINT " Steer with the"
LOCATE 4, 5: COLOR 14: PRINT "LEFT/RIGHT arrows.";: COLOR 9: PRINT " Press the ";
COLOR 14: PRINT "SPACE BAR";: COLOR 9: PRINT " to deploy or stow the spinnaker."
LOCATE 6, 33: COLOR 12: PRINT "Starting off"
COLOR 3
LOCATE 7, 5: PRINT "Adjust the sail ";: COLOR 14: PRINT "(up/down arrows)";: COLOR 3: PRINT " to catch the wind. If you are headed too"
LOCATE 8, 5: PRINT "steeply into the wind, steer";: COLOR 14: PRINT " (left/right arrows)";: COLOR 3: PRINT " away from it to speed up."
LOCATE 9, 5: PRINT "If the sail is ";: COLOR 8: PRINT "flapping in the wind,";: COLOR 3: PRINT " pull in the boom. If stopped and"
LOCATE 10, 5: PRINT "headed straight into the wind, use full rudder to slowly turn away."
LOCATE 12, 29: COLOR 12: PRINT "Sailing with sidewinds"
COLOR 9
LOCATE 13, 5: PRINT "Sailing east-west with the sail adjusted for";: COLOR 11: PRINT " laminar flow,";: COLOR 3: PRINT " as you sail"
LOCATE 14, 5: COLOR 9: PRINT "faster, the apparent wind moves forward, and the boom goes ";: COLOR 12: PRINT "orange";: COLOR 3: PRINT " again."
LOCATE 15, 5: COLOR 9: PRINT "Pull the sail in a bit more to keep ";: COLOR 11: PRINT "laminar flow."
LOCATE 17, 32: COLOR 12: PRINT "Sailing downwind"
COLOR 3
LOCATE 18, 5: PRINT "As soon as you turn south a little, the wind will start coming from the"
LOCATE 19, 5: PRINT "rear side, so let out the sail. Here you can press the";: COLOR 14: PRINT " SPACE BAR ";: COLOR 3: PRINT "to"
LOCATE 20, 5: PRINT "deploy the";: COLOR 14: PRINT " Spinnaker, ";: COLOR 3: PRINT "the large baloon sail, and your yacht will reach"
LOCATE 21, 5: PRINT "its maximum speed with";: COLOR 11: PRINT " laminar flow.";: COLOR 3: PRINT " This is however not possible"
LOCATE 22, 5: COLOR 3: PRINT "when sailing directly downwind, as the wind only pushes from behind."
LOCATE 24, 33: COLOR 12: PRINT "Sailing upwind"
COLOR 9
LOCATE 25, 5: PRINT "To sail sharper into the wind, pull in the sail. The sharpest effective"
LOCATE 26, 5: PRINT "sailing angle is 30 degrees, but speed is low. 40 or 45 is better if you"
LOCATE 27, 5: PRINT "have to tack or zig-zag your way NORTH."
LOCATE 28, 62: COLOR 2: PRINT "Any key: Page 3"
pausa$ = INPUT$(1)
CLS

LINE (10, 6)-(627, 457), 12, B
LINE (11, 7)-(626, 456), 14, B
SOUND 500, .8
SOUND 800, .8
LOCATE 2, 28: COLOR 12: PRINT "The two race courses                            3"
LOCATE 3, 5: COLOR 14: PRINT "The first race:";: COLOR 2: PRINT " `The Lazy Eight'.";: COLOR 7: PRINT " It has two slanted";: COLOR 11: PRINT " upwind";: COLOR 7: PRINT " legs which"
LOCATE 4, 5: PRINT "need no tacking, and two legs that are almost straight";: COLOR 12: PRINT " downwind."
LOCATE 6, 5: COLOR 14: PRINT "The second race:";: COLOR 2: PRINT " `The Triangle',";: COLOR 7: PRINT " with an";: COLOR 11: PRINT " upwind";: COLOR 7: PRINT " leg, where tacking will"
LOCATE 7, 5: PRINT "be necessary, followed by a slanted";: COLOR 12: PRINT " downwind";: COLOR 7: PRINT " leg, and a sidewind leg."
LOCATE 9, 34: COLOR 12: PRINT "Racing"
COLOR 9
LOCATE 10, 5: COLOR 9: PRINT "Both races start and finish at the SOUTH EAST buoy. At the start, touch"
LOCATE 11, 5: PRINT "or sail into the";: COLOR 4: PRINT " red";: COLOR 9: PRINT " circle at the first buoy. Correct your heading and"
LOCATE 12, 5: PRINT "boom position, and head for the new buoy marked with the";: COLOR 4: PRINT " red ";: COLOR 9: PRINT "circle. Sail"
LOCATE 13, 5: PRINT "round that buoy, touching or entering the circle, and head for the next"
LOCATE 14, 5: PRINT "buoy with the";: COLOR 4: PRINT " red";: COLOR 9: PRINT " circle. Buoys can be rounded in any direction. The lap"
LOCATE 15, 5: PRINT "ends behind the first buoy.";: COLOR 7: PRINT " The race ends when you finish the last lap."
LOCATE 17, 32: COLOR 12: PRINT "Special notes"
LOCATE 18, 5: COLOR 3: PRINT "Be careful overtaking: Downwind near the other yacht is the wind shadow,"
LOCATE 19, 5: PRINT "so you will lose speed. Upwind, you can `steal the wind'. Accidents with"
LOCATE 20, 5: PRINT "the opponent will throw both yachts off course with considerable loss of"
LOCATE 21, 5: PRINT "speed, and you may have to give up and start a new race from the menu."
LOCATE 22, 5: PRINT "However, buoys are no hazard as they are made of inflatable plastic."
LOCATE 23, 5: COLOR 9: PRINT "Also remember to keep your";: COLOR 14: PRINT " Spinnaker ";:
COLOR 9: PRINT "stowed when you are ";: COLOR 5: PRINT "reaching";: COLOR 9: PRINT " or "
LOCATE 24, 5: COLOR 5: PRINT "close hauled,";: COLOR 9: PRINT " because otherwise you will come to a stop!"
LOCATE 25, 5: COLOR 7: PRINT "A warning sound will notify you of these three situations."
LOCATE 27, 31: COLOR 11: PRINT "HAPPY SAILING !"
LOCATE 28, 56: COLOR 2: PRINT "Any key: Back to Menu"
pausa$ = INPUT$(1)
CLS
RETURN

racevals:
REM -------------- Opponent's attitude / speed (1-5) -------------
oppvel = 1
opponent$(1) = "Simply Laid Back"
opponent$(2) = "Quite Easy Going"
opponent$(3) = "Fairly Concerned"
opponent$(4) = "Rather Motivated"
opponent$(5) = "Definitely Brisk"

oppspeed(1) = .0038
oppspeed(2) = .0041
oppspeed(3) = .0044
oppspeed(4) = .0047
oppspeed(5) = .005

REM ------- Set Race Type: 1-3
difficulty$(1) = "1. `The Lazy Eight'"
difficulty$(2) = "2. `The Triangle'  "
r = 1
curs = 18
laps = 1
numlap$(1) = "one"
numlap$(2) = "two"
numlap$(3) = "three"
numlap$(4) = "four"
numlap$(5) = "five"
RETURN


help:

COLOR 9
LOCATE 12, 30: PRINT "Your Sailing Monitor says:  "
LOCATE 13, 30: PRINT "--------------------------  "

IF boomcol = 11 AND ve > 0 THEN
    LOCATE 14, 30: COLOR 2: PRINT "Your sail is catching the   "
    LOCATE 15, 30: COLOR 2: PRINT "wind very efficiently. You  "
    LOCATE 16, 30: COLOR 2: PRINT "are getting laminar flow.   "
END IF
IF ABS(relwind) < 30 THEN
    LOCATE 18, 30: COLOR 13: PRINT "You are headed very steeply "
    LOCATE 19, 30: COLOR 13: PRINT "upwind. Turn away from the  "
    LOCATE 20, 30: COLOR 13: PRINT "wind to increase your speed."
END IF

IF boomcol = 8 THEN
    LOCATE 14, 30: COLOR 12: PRINT "Your sail is flapping in the"
    LOCATE 15, 30: COLOR 12: PRINT "wind. Pull the loose rope on"
    LOCATE 16, 30: COLOR 12: PRINT "the sailboom tight, and pull"
    LOCATE 17, 30: COLOR 12: PRINT "in the sail (down arrow).  "
END IF

IF boomcol = 12 AND ABS(relwind) < 141 THEN
    LOCATE 14, 30: COLOR 13: PRINT "The wind is only pushing the"
    LOCATE 15, 30: COLOR 13: PRINT "sail. Let out the sail a bit"
    LOCATE 16, 30: COLOR 13: PRINT "(up-arrow) to get laminar   "
    LOCATE 17, 30: COLOR 13: PRINT "flow and go faster.         "
END IF

IF boomcol = 12 AND ABS(relwind) > 140 THEN
    LOCATE 14, 30: COLOR 13: PRINT "The wind is only pushing the"
    LOCATE 15, 30: COLOR 13: PRINT "sail. This is OK for very   "
    LOCATE 16, 30: COLOR 13: PRINT "rear winds. Let out the sail"
    LOCATE 17, 30: COLOR 13: PRINT "completely (up-arrow).      "
END IF


IF ABS(relwind) > 90 AND spi = -1 THEN
    LOCATE 18, 30: COLOR 14: PRINT "With rear wind, deploy the  "
    LOCATE 19, 30: COLOR 14: PRINT "Spinnaker for extra speed.  "
    LOCATE 20, 30: COLOR 14: PRINT "Press Space Bar to deploy or"
    LOCATE 21, 30: COLOR 14: PRINT "stow the Spinnaker.         "
END IF

IF ABS(relwind) < 90 AND spi = 1 THEN
    LOCATE 14, 30: COLOR 12: PRINT "Stow the Spinnaker! It's for"
    LOCATE 15, 30: COLOR 12: PRINT "rear winds only, or it  will"
    LOCATE 16, 30: COLOR 14: PRINT "stop you. Press Space Bar to"
    LOCATE 17, 30: COLOR 14: PRINT "stow or deploy Spinnaker.   "
END IF

IF reduce < 1 THEN
    LOCATE 14, 30: COLOR 12: PRINT "You are in your opponent's  "
    LOCATE 15, 30: COLOR 12: PRINT "wind shadow, loosing wind.  "
    LOCATE 16, 30: COLOR 13: PRINT "From the other side, you can"
    LOCATE 17, 30: COLOR 13: PRINT "steal your opponent's wind. "
END IF


LOCATE 22, 30: COLOR 6: PRINT ">>> ENTER to continue <<< "

DO
    EnterOn = _KEYDOWN(Key_Enter&)
LOOP UNTIL EnterOn

FOR lini = 12 TO 22: LOCATE lini, 30: PRINT STRING$(31, " "): NEXT lini

RETURN

